% Copyright (C) 2015 by Daniel Kraft <d@domob.eu>
% OctConf 2015 - Octave's profiler

%   CHI = floodfill (IMG, A, B, R, C, MAXDIST = Inf)
%
% Perform a floodfill procedure on the image IMG, including all pixels
% with intensity between A and B starting at (R, C).  Returned is the
% final segment's characteristic function as logical array.
%
% If MAXSTEPS is set, it is the maximum distance (in L1 terms, i. e.,
% "flooding steps") to fill.

function chi = floodfill (img, a, b, r, c, maxDist = Inf)
  n = size (img, 1);
  assert (n, size (img, 2));

  chi = false (size (img));
  chi(r, c) = true;
  assert (img(r, c) >= a && img(r, c) <= b);

  curDist = 0;
  while (curDist < maxDist)
    up = [chi(2 : end, :); false(1, n)];
    down = [false(1, n); chi(1 : end - 1, :)];
    right = [false(n, 1), chi(:, 1 : end - 1)];
    left = [chi(:, 2 : end), false(n, 1)];
    chip = ((chi | up | down | left | right) & (img >= a & img <= b));

    assert (all (chip(chi)));
    if (all (chi(chip)))
      assert (chi, chip);
      return;
    endif

    chi = chip;
    ++curDist;
  endwhile
endfunction
