% Copyright (C) 2015 by Daniel Kraft <d@domob.eu>
% OctConf 2015 - Octave's profiler

%   IMG = getImage (N, NL)
%
% Construct an example grey-scale image of size NxN to use for the demo.
% Gaussian noise of standard deviation NL is applied.

function img = getImage (n, nl)
  x = linspace (0, 1, n);
  [XX, YY] = meshgrid (x, x);

  sphere1 = hypot (XX - 0.3, YY - 0.8) - 0.1;
  sphere2 = hypot (XX - 0.6, YY - 0.4) - 0.3;
  inner = hypot (XX - 0.57, YY - 0.4) - 0.1;
  shape = min (sphere1, max (sphere2, -inner));

  sharpness = 20;
  img = (pi / 2 + atan (sharpness * shape)) / pi;

  img += nl * randn (n, n);
  minVal = min (img(:));
  maxVal = max (img(:));
  img = (img - minVal) / (maxVal - minVal);
endfunction
