% Copyright (C) 2015 by Daniel Kraft <d@domob.eu>
% OctConf 2015 - Octave's profiler

%   L = laplacian (INSIDE)
%
% Generate a finite-difference (negative) Laplacian with homogeneous Neumann
% boundary conditions.  The inside of the domain (for constructing the
% boundary conditions) is marked by the logical array INSIDE.

function L = laplacian (inside)
  n = size (inside, 1);
  assert (n, size (inside, 2));

  L = sparse (n^2, n^2);

  linearise = @(k, l) k + (l - 1) * n;
  deltas = [-1, 0; 1, 0; 0, -1; 0, 1];

  for r = 1 : n
    for c = 1 : n
      ind = linearise (r, c);

      neighbours = 0;
      for i = 1 : size (deltas, 1)
        rp = r + deltas(i, 1);
        cp = c + deltas(i, 2);

        neigh = linearise (rp, cp);
        if (rp >= 1 && rp <= n && cp >= 1 && cp <= n && inside(neigh))
          ++neighbours;
          assert (L(ind, neigh) == 0);
          L(ind, neigh) = -1;
        endif
      endfor

      assert (L(ind, ind) == 0);
      L(ind, ind) = neighbours;
    endfor
  endfor

  assert (all (sum (L, 2) == 0));
endfunction
