% Copyright (C) 2015 by Daniel Kraft <d@domob.eu>
% OctConf 2015 - Octave's profiler

%   L = laplacian_2 (INSIDE)
%
% Vectorised version of laplacian.

function L = laplacian_2 (inside)
  n = size (inside, 1);
  assert (n, size (inside, 2));

  % Start out by building the base Laplacian without considering the domain.
  % This already cares for "out-of-bounds" neighbour indices.
  diags = repmat ([-1, -1, -1, -1], n^2, 1);
  diags(n : n : end, 2) = 0;
  diags(n + 1 : n : end, 3) = 0;
  L = spdiags (diags, [-n, -1, 1, n], n^2, n^2);

  % Remove all entries that represent neighbours of "outside" cells.
  outsiders = !inside;
  up = [outsiders(2 : end, :); zeros(1, n)];
  down = [zeros(1, n); outsiders(1 : end - 1, :)];
  right = [zeros(n, 1), outsiders(:, 1 : end - 1)];
  left = [outsiders(:, 2 : end), zeros(n, 1)];
  L = cancelShift (L, n, up, 1);
  L = cancelShift (L, n, down, -1);
  L = cancelShift (L, n, left, n);
  L = cancelShift (L, n, right, -n);

  % Build the diagonal based on the off-diagonal entries (to ensure
  % mass conservation).  This, of course, prevents us from *checking*
  % mass conservation as an additional error check, but it makes life
  % easier.  We compare to laplacian anyway, so should be able
  % to catch bugs.
  mainDiag = sum (L, 2);
  L = spdiags (-mainDiag, 0, L);

  %assert (L, laplacian (inside));
endfunction

% Helper routine to cancel out neighbour entries corresponding to a shift.
function L = cancelShift (L, n, shifted, offs)
  centres = find (shifted);
  neighbours = centres + offs;
  assert (all (neighbours >= 1 & neighbours <= n^2));
  L(centres, neighbours) = 0;
endfunction
