% Copyright (C) 2015 by Daniel Kraft <d@domob.eu>
% OctConf 2015 - Octave's profiler

%   U = smoothImage (IMG, WHERE, H, BETA)
%
% Apply Gaussian smoothing (by inverting the Laplacian) to the image IMG,
% where the Laplacian is constructed on the domain WHERE.  BETA is the
% smoothing parameter used and H the grid spacing.

function u = smoothImage (img, where, h, beta)
  L = laplacian (where) / h^2;
  Lr = L(where(:), where(:));
  imgr = img(where(:));

  ur = (beta * Lr + eye (size (Lr))) \ imgr;
  u = NA (size (img));
  u(where) = ur;
endfunction
